# -*- coding: UTF-8 -*-

# NVDA appModule for Winboard
# Develope Version
#This file is covered by the GNU General Public License.
#See the file COPYING.txt for more details.
#Copyright (C) 2018 Javi Dominguez <fjavids@gmail.com>

import appModuleHandler
import api
import controlTypes
import ui
import re
from threading import Timer
import tones

class AppModule(appModuleHandler.AppModule):

	def __init__(self, *args, **kwargs):
		super(AppModule, self).__init__(*args, **kwargs)
		self.terminalLinesCount = 17
		self.lastTerminalMessage = ""
		self.reviewLine = 1

	def event_valueChange(self, obj, next):
		fg = api.getForegroundObject()
		if obj == fg.getChild(0): # controlTypes.STATE_READONLY in obj.states:
			self.readTerminal(1)
		next()

	def script_readTerminal(self, gesture):
		self.readTerminal(1, True)
		self.reviewLine = 0

	def script_readPrevious(self, gesture):
		try:
			flag = False
			while not flag:
				flag = self.readTerminal(self.reviewLine+1, True)
				self.reviewLine = self.reviewLine+1
		except IndexError:
			tones.beep(300,80)

	def script_readNext(self, gesture):
		flag = False
		while not flag:
			flag = self.readTerminal(self.reviewLine-1, True)
			self.reviewLine = self.reviewLine-1

	def readTerminal(self, line = 1, allowRepeat = False):
		fg = api.getForegroundObject()
		try:
			terminal = fg.getChild(0).windowText
		except AttributeError:
			focus = api.getFocusObject()
			if focus.parent.next.getChild(3).firstChild.firstChild.windowClassName == u'RICHEDIT':
				terminal = focus.parent.next.getChild(3).firstChild.firstChild.windowText
			else:
				terminal = ""
		lines = terminal.split(r"fics%")[2:]
		if line < 1:
			tones.beep(200,80)
			self.reviewLine = 1
			return True
		lines.reverse()
		if lines:
			lastMessage = "\n".join(lines[line].split("\n")[1:])
		else:
			if allowRepeat:
				tones.beep(200, 90)
				return True
			lines = terminal.split("\n")
			ui.message("\n".join(lines[self.terminalLinesCount:]))
			self.terminalLinesCount = len(lines)
			return
		if not lastMessage: return False
		if lastMessage != self.lastTerminalMessage or allowRepeat:
			ui.message(lastMessage)
		self.lastTerminalMessage = lastMessage
		return True

	def script_readChat(self, gesture):
		fg = api.getForegroundObject()
		lines = fg.getChild(0).windowText.split("\n")
		lines.reverse()
		for l in lines:
			if re.match(".*says:.*|.*tells you:.*", l):
				ui.message(l)
				break

	__gestures = {
	"kb:NVDA+-": "readTerminal",
	"kb:NVDA+shift+-": "readChat",
	"kb:NVDA+leftArrow": "readPrevious",
	"kb:NVDA+rightArrow": "readNext"
}
